import {
  emailValidator,
  identityCardValidator,
  mobilePhoneValidator,
  sqlSimpleValidator
} from '@cip/utils/form-validator'
import defaultInputComponentConfig from '../cip-form-input/component-config'
import defaultLayoutComponentConfig from '../cip-form-layout/component-config'
import { defineAsyncComponent } from 'vue'

// 可以转换的validateType
const validatorMap = {
  email: emailValidator,
  identityCard: identityCardValidator,
  mobilePhone: mobilePhoneValidator,
  sql: sqlSimpleValidator
}

const isInputType = (config) => {
  if (config.triggerType === 'input') { return true }
  // 兼容老的未设置triggerType的表单数据验证
  return ['input', 'textarea', 'number', 'numberRange', 'table', 'password', undefined].includes(config.type)
}
// 获取单字段规则
export const getRulesByFieldConfig = (config, otherValue, dependOnValues) => {
  const rules = []
  if (config.required) { // 必填
    const defaultPreText = isInputType(config) ? '请输入' : '请选择'
    const requiredMessage = config.requiredErrorMessage || `${defaultPreText}${config.label}`
    const requiredRule = { required: true, message: requiredMessage }
    if (config.requiredType) {
      requiredRule.type = config.requiredType
    }
    rules.push(requiredRule)
    if (config.type === 'dateRange' && config.otherKey) { // 日期范围
      const validator = (rule, value, cb) => {
        if (value) {
          if (otherValue) {
            cb()
          } else {
            cb(new Error(requiredMessage))
          }
        } else {
          cb()
        }
      }
      const otherRules = { validator }
      rules.push(otherRules)
    }
  }
  // 提取公共函数
  const pushValidatorFunction = (validator, type) => {
    const message = config.validateValueErrorMessage
    const rule = {
      validator,
      trigger: type || 'blur'
    }
    if (message) {
      rule.message = message
    }
    return rule
  }
  if (config.validateValue && validatorMap[config.validateValue]) { // 内置值类型校验
    const validator = validatorMap[config.validateValue]
    if (validator) {
      rules.push(pushValidatorFunction(validator))
    }
  }
  // 正则校验
  if (config.regexpValidate) {
    try {
      const reg = new RegExp(config.regexpValidate)

      const rule = {
        pattern: reg,
        message: config.regexpValidateErrorMessage || `未能通过${reg.toString()}校验`,
        trigger: 'blur'
      }
      rules.push(rule)
    } catch (e) { console.log(e.message) }
  }
  // 远程校验值是否已经存在
  if (config.validateExistRemote) {
    const validator = async (rule, value, callback) => {
      const { data } = await config.validateExistRemote(value, dependOnValues)
      console.log(data)
      if (data) {
        callback(new Error(rule.message || '已存在'))
      } else {
        callback()
      }
    }
    rules.push(pushValidatorFunction(validator))
  }
  return rules
}

const generateAppendKey = (append) => {
  return append ? `-${append}` : ''
}
const analyseComponents = (dictionary, append) => {
  const result = {}
  const appendKey = generateAppendKey(append) // append ? `-${append}` : ''
  const appendPath = append ? `/${append}` : ''
  const errorComponent = defineAsyncComponent(dictionary.default(appendPath))
  Object.keys(dictionary).map(key => {
    const loader = dictionary[key](appendPath) || dictionary.default(appendPath)
    result[`${key}${appendKey}`] = defineAsyncComponent({
      loader,
      errorComponent,
      onError (error, retry, fail, attempts) {
        console.log(error)
        console.warn(`${key}${appendKey}组件加载失败，加载默认组件`)
        fail()
      }
    })
  })
  return result
}

const getComponentDictionary = (componentConfig) => {
  const result = {}
  Object.keys(componentConfig).forEach(key => {
    const config = componentConfig[key]
    const component = typeof config === 'function' ? config : componentConfig[key].component
    result[key] = component
  })
  return result
}
const getLayoutType = (componentConfig) => {
  const result = []
  Object.keys(componentConfig).forEach(key => {
    if (componentConfig[key].layout) {
      result.push(key)
    }
  })
  return result
}

/**
 * 贯穿整个数据渲染的单例对象
 */
export class DRender {
  constructor () {
    if (!DRender.instance) {
      this.init()
      DRender.instance = this
    }
    return DRender.instance
  }

  defaultComponentConfig={}
  componentDictionary = {}
  layoutTypeList=[]
  // 初始化
  init () {
    const defaultComponentConfig = { ...defaultInputComponentConfig, ...defaultLayoutComponentConfig }
    this.defaultComponentConfig = defaultComponentConfig
    this.componentDictionary = getComponentDictionary(defaultComponentConfig)
    this.layoutTypeList = getLayoutType(defaultComponentConfig)
  }

  setConfig (renderConfig = {}) {
    const { components, plugins } = renderConfig
    const customComponentConfig = {}
    if (components) Object.assign(customComponentConfig, components)
    if (plugins) Object.assign(customComponentConfig, ...plugins)
    this.setCustomComponents(customComponentConfig)
  }

  setCustomComponents (customComponentsConfig) {
    const componentConfig = { ...this.defaultComponentConfig, ...customComponentsConfig }
    const componentDictionary = getComponentDictionary(componentConfig)
    this.componentDictionary = componentDictionary
    this.layoutTypeList = getLayoutType(componentConfig)
  }

  // 获取组件
  getComponent (type, append = '') {
    if (!this[`${append}Components`]) {
      // 如果是没有缓存过的组件组 分析一次
      this[`${append}Components`] = analyseComponents(this.componentDictionary, append)
    }
    return this[`${append}Components`][`${type}${generateAppendKey(append)}`]
  }

  isLayoutType (type) {
    return this.layoutTypeList.includes(type)
  }
}

// 必须直接实例化以防止数据为初始化导致的错误
const dRender = new DRender()
// 根据d-renderConfig
let dRenderConfig
try {
  dRenderConfig = require('../../../d-render.config').default
  dRender.setConfig(dRenderConfig)
} catch (e) {
  process.env.NODE_ENV === 'development' && console.log(e)
}
